from fastapi import APIRouter, Request, Depends, Security
from fastapi.encoders import jsonable_encoder
from fastapi_pagination import Params
from api.db.database import get_db
from api.v1.services.user import user_service
from api.v1.routes.task import available_tasks, get_task, fetch_mytasks, get_task_by_id, get_user_submissions, user_task_statistics, task_partner_statistics
from api.v1.services.coin_token import token_services

from sqlalchemy.orm import Session

from fastapi.responses import HTMLResponse, RedirectResponse
from api.utils.config import templates_env

dashview = APIRouter(prefix="", tags=["Dashboard VIews and UI"])

Title = "Dashboard"

def dashboard_items(request: Request, db: Session = Depends(get_db)):
    user_id = user_service.fetch_user_refresh_token(request, db=db)

    if user_id == None:
        return None
    user = user_service.get_user_by_id(db=db, id=user_id)
    wallet = user_service.fetch_user_walletbalance(user_id, db=db)
    return {"user": user, "balance": wallet, "user_id": user_id}


# User Dashboard View
@dashview.get("/dashboard", response_class=HTMLResponse)
async def dashboard(request: Request, db: Session = Depends(get_db)):
    user_id = user_service.fetch_user_refresh_token(request, db=db)
    if user_id == None:
        return RedirectResponse("/login")
    user = user_service.get_user_by_id(db=db, id=user_id)
    wallet = user_service.fetch_user_walletbalance(user_id, db=db)
    token_balances = await token_services.user_token_balances(db, user_id=user_id)
    tokens = token_balances.get("total_value",[])
    task_stats = user_task_statistics(db=db, user_id=user.id)
    template = templates_env.get_template("main.html")
    # print(wallet, user)
    return template.render(
        {
            "request": request,
            "title": Title,
            "tag" : "My Account",
            "page": "dashboard",
            "user": user,
            "data": {"balance": tokens, "user": user,
                     "task_stats": task_stats},
            "templates_env": templates_env,
            "isLogin": True
        }
    )


# User Change Password View
@dashview.get("/dashboard-password", response_class=HTMLResponse)
def reset_password(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Password Reset View"""
    # print(items)
    template = templates_env.get_template("main.html")
    return template.render(
        {
         "request": request, 
         "page": "dashboard-password", 
         "title": Title, 
         "tag" : "Change Password",
         "data" :{"user": items["user"]},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

def user_role(request, db):
    items = dashboard_items(request, db=db)
    user_detail = items['user']
    if user_detail.role == 'Reply Guy':
        role = 'replyguy'
    elif user_detail.role == 'KOL':
        role = 'kol'

    return role

# User jobs View
@dashview.get("/dashboard-jobs", response_class=HTMLResponse)
async def view_jobs(request: Request, db: Session = Depends(get_db),  params: Params=Depends()):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    
    role = user_role(request, db)
    template = templates_env.get_template("main.html")
    tasks = await available_tasks(role, db=db, params=params,
    
    #current_user = Security(user_service.get_current_user)
    )

    return template.render(
        {
         "request": request, 
         "page": "dashboard-jobs", 
         "title": Title, 
         "tag" : "Available Tasks",
         "data" :{"user": items["user"], "role":role, "tasks":tasks.get('data'),},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@dashview.get("/my-tasks", response_class=HTMLResponse)
async def view_myjobs(request: Request, db: Session = Depends(get_db),  params: Params=Depends()):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    
    role = user_role(request, db)
    template = templates_env.get_template("main.html")
    items = dashboard_items(request, db=db)
    user_detail = items['user']
    current_user = user_detail.id
    tasks = await fetch_mytasks(
        current_user,
        role, 
        db=db, 
        params=params,
    
    #current_user = Security(user_service.get_current_user)
    )
    sorted_task = sorted(tasks.get('data'), 
                         key=lambda x: x["created_at"], 
                         reverse=True)

    return template.render(
        {
         "request": request, 
         "page": "my-task", 
         "title": Title, 
         "tag" : "Task you're assigned",
         "data" :{"user": items["user"], "role":role, "tasks":sorted_task,},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


@dashview.get("/submit", response_class=HTMLResponse)
def submit_task(
    #params, 
    #current_user,
    request: Request, 
    db: Session = Depends(get_db) 
   ):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Password Reset View"""
    # print(items)
    template = templates_env.get_template("main.html")
    #my_task = get_task(db, params, current_user)
    #if not my_task:
    #    task_stat = "Oops, It seem task is unavailable, \nTry again!"
    role = user_role(request, db)
    return template.render(
        {
         "request": request, 
         "page": "submit-task", 
         "title": Title, 
         "tag" : "Create Task",
         "data" :{"user": items["user"], 'role': role,},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


@dashview.get("/{task_id}", response_class=HTMLResponse)
async def view_details(task_id, request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    
    template = templates_env.get_template("main.html")
    items = dashboard_items(request, db=db)
    tasks = await get_task_by_id(
        task_id, 
        db=db,
        current_user = Security(user_service.get_current_user)
    )
    tasks = jsonable_encoder(tasks)
    role = user_role(request, db)
    return template.render(
        {
         "request": request, 
         "page": "task-details", 
         "title": "Task", 
         "tag" : "details",
         "data" :{"user": items["user"], "role": role,"task": tasks},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@dashview.get("/dashboard/withdraw", response_class=HTMLResponse)
async def withdraw(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Password Reset View"""
    # print(items)
    user_id = items["user_id"]
    template = templates_env.get_template("main.html")
    listed_token = await token_services.listed_tokens(
        db)
    listed_tokens = jsonable_encoder(listed_token)
    token_balances = await token_services.user_token_balances(db, user_id=user_id)
    tokens = token_balances.get("balances",[])
    return template.render(
        {
         "request": request, 
         "page": "withdrawal", 
         "title": Title, 
         "tag" : "Deposit & Withdrawal",
         "data" :{"user": items["user"], "token": tokens, "token_list": listed_tokens,},  
         "user": items["user"],
         
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@dashview.get("/dashboard/transactions", response_class=HTMLResponse)
def reset_password(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Password Reset View"""
    # print(items)
    user_id = items["user_id"]
    template = templates_env.get_template("main.html")
    transactions = token_services.all_user_transactions(db, user_id=user_id)
    return template.render(
        {
         "request": request, 
         "page": "transactions", 
         "title": Title, 
         "tag" : "All transactions",
         "data" :{"user": items["user"], "transactions": transactions},  
         "user": items["user"],
         
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


@dashview.get("/submissions/{task_id}", response_class=HTMLResponse)
async def submissions(
    task_id, 
    request: Request, 
    db: Session = Depends(get_db),
):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    role = user_role(request, db)
    template = templates_env.get_template("main.html")
    #users = await get_users(db=db,page=1,per_page=10,is_active=None, is_deleted=None,is_verified=None, is_super_admin=None)
    user = items["user"]
    tasks = await get_user_submissions(db=db, task_id=task_id, role=role,
    current_user = user)
    """ print(user.id)
    print(tasks) """
    if not tasks:
        tasks = []
    return template.render(
        {
         "request": request, 
         "page": "submissions", 
         "title": Title, 
         "tag" : "All Tasks",
         "data" :{"tasks":tasks.get("submissions"),},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


@dashview.get("/setting/profile", response_class=HTMLResponse)
def reset_password(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    
    user = items["user"]
    base_url = str(request.base_url).rstrip("/")
    profile_url = f"{base_url}/profile/{user.username}"
    template = templates_env.get_template("main.html")
    return template.render(
        {
         "request": request, 
         "page": "profile-settings", 
         "title": Title, 
         "tag" : "Profile Settings",
         "data" :{"user": items["user"],
                  "profile_url": profile_url,
                  },  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )

@dashview.get("/ticket/support", response_class=HTMLResponse)
def reset_password(request: Request, db: Session = Depends(get_db)):
    items = dashboard_items(request, db=db)
    if items == None:
        return RedirectResponse("/login")
    """Password Reset View"""
    # print(items)
    template = templates_env.get_template("main.html")
    return template.render(
        {
         "request": request, 
         "page": "support-ticket", 
         "title": Title, 
         "tag" : "Report an Issue",
         "data" :{"user": items["user"]},  
         "user": items["user"],
         "templates_env": templates_env,
         "isLogin": True
         
         }
    )


